@@ -36,7 +36,6 @@
 #include "dtd.h"
 
 #include <misc/loader.h>
-#include <misc/htmlhashes.h>
 #include <khtmlview.h>
 #include <khtml_part.h>
 #include <xml/dom_docimpl.h>
@@ -706,7 +705,7 @@ void ProspectiveTokenizer::processAttribute()
         DOMStringImpl attrDS(DOMStringImpl::ShallowCopy, m_attributeName.data(), m_attributeName.size());
         LocalName attrLocal = LocalName::fromString(&attrDS, IDS_NormalizeLower);
         uint attribute = attrLocal.id();
-        if (attribute == ATTR_SRC && m_urlToLoad.isEmpty())
+        if (attribute == localNamePart(ATTR_SRC) && m_urlToLoad.isEmpty())
             m_urlToLoad = parseURL(DOMString(m_attributeValue.data(), m_attributeValue.size()));
         break;
     }
@@ -715,11 +714,11 @@ void ProspectiveTokenizer::processAttribute()
         DOMStringImpl attrDS(DOMStringImpl::ShallowCopy, m_attributeName.data(), m_attributeName.size());
         LocalName attrLocal = LocalName::fromString(&attrDS, IDS_NormalizeLower);
         uint attribute = attrLocal.id();
-        if (attribute == ATTR_HREF && m_urlToLoad.isEmpty())
+        if (attribute == localNamePart(ATTR_HREF) && m_urlToLoad.isEmpty())
             m_urlToLoad = parseURL(DOMString(m_attributeValue.data(), m_attributeValue.size()));
-        else if (attribute == ATTR_REL) {
-            QString val = QString::fromRawData(m_attributeValue.data(), m_attributeValue.size());
-            m_linkIsStyleSheet = val.contains("styleSheet") && !val.contains("alternate") && !val.contains("icon");
+        else if (attribute == localNamePart(ATTR_REL)) {
+            QString val = DOMStringImpl(DOMStringImpl::ShallowCopy, m_attributeValue.data(), m_attributeValue.size()).lower()->string();
+            m_linkIsStyleSheet = val.contains("stylesheet") && !val.contains("alternate") && !val.contains("icon");
         }
     }
     default:
@@ -867,7 +866,7 @@ void ProspectiveTokenizer::emitTag()
     if (o)
         m_document->docLoader()->registerPreload( o );
 
-    m_urlToLoad = QString();
+    m_urlToLoad = DOMString();
     m_linkIsStyleSheet = false;
 }
     
