@@ -31,13 +31,9 @@
 #include <kio/job.h>
 #include <kio/jobuidelegate.h>
 #include <kmessagebox.h>
-#include <kglobal.h>
-#include <kglobalsettings.h>
 #include "kprotocolmanager.h"
 #include "kmountpoint.h"
-#include <sys/stat.h>
 
-#include <assert.h>
 #include <QFile>
 
 // Enable this to get printDebug() called often, to see the contents of the cache
@@ -58,12 +54,12 @@ KDirListerCache::KDirListerCache()
   connect( &pendingUpdateTimer, SIGNAL(timeout()), this, SLOT(processPendingUpdates()) );
   pendingUpdateTimer.setSingleShot( true );
 
-  connect( KDirWatch::self(), SIGNAL( dirty( const QString& ) ),
-           this, SLOT( slotFileDirty( const QString& ) ) );
-  connect( KDirWatch::self(), SIGNAL( created( const QString& ) ),
-           this, SLOT( slotFileCreated( const QString& ) ) );
-  connect( KDirWatch::self(), SIGNAL( deleted( const QString& ) ),
-           this, SLOT( slotFileDeleted( const QString& ) ) );
+  connect( KDirWatch::self(), SIGNAL(dirty(QString)),
+           this, SLOT(slotFileDirty(QString)) );
+  connect( KDirWatch::self(), SIGNAL(created(QString)),
+           this, SLOT(slotFileCreated(QString)) );
+  connect( KDirWatch::self(), SIGNAL(deleted(QString)),
+           this, SLOT(slotFileDeleted(QString)) );
 
   kdirnotify = new org::kde::KDirNotify(QString(), QString(), QDBusConnection::sessionBus(), this);
   connect(kdirnotify, SIGNAL(FileRenamed(QString,QString)), SLOT(slotFileRenamed(QString,QString)));
@@ -125,15 +121,15 @@ bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
       // (the non-unicity of the stringlist gives us the refcounting, basically).
   }
 
-  if (!validUrl(lister, _url)) {
+    if (!validUrl(lister, _url)) {
         kDebug(7004) << lister << "url=" << _url << "not a valid url";
         return false;
-  }
+    }
 
+    //kDebug(7004) << lister << "url=" << _url << "keep=" << _keep << "reload=" << _reload;
 #ifdef DEBUG_CACHE
     printDebug();
 #endif
-  //kDebug(7004) << lister << "url=" << _url << "keep=" << _keep << "reload=" << _reload;
 
     if (!_keep) {
         // stop any running jobs for lister
@@ -145,7 +141,7 @@ bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
         lister->d->rootFileItem = KFileItem();
     } else if (lister->d->lstDirs.contains(_url)) {
         // stop the job listing _url for this lister
-        stop(lister, _url, true /*silent*/);
+        stopListingUrl(lister, _url, true /*silent*/);
 
         // remove the _url as well, it will be added in a couple of lines again!
         // forgetDirs with three args does not do this
@@ -181,9 +177,11 @@ bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
             if (itemU) {
                 kDebug(7004) << "Entry already in use:" << _url;
                 // if _reload is set, then we'll emit cached items and then updateDirectory.
+                if (lister->d->autoUpdate)
+                    itemU->incAutoUpdate();
             } else {
                 kDebug(7004) << "Entry in cache:" << _url;
-                itemFromCache->decAutoUpdate();
+                // In this code path, the itemsFromCache->decAutoUpdate + itemU->incAutoUpdate is optimized out
                 itemsInUse.insert(urlStr, itemFromCache);
                 itemU = itemFromCache;
             }
@@ -192,7 +190,7 @@ bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
 
             // List items from the cache in a delayed manner, just like things would happen
             // if we were not using the cache.
-            new KDirLister::Private::CachedItemsJob(lister, itemU->lstItems, itemU->rootItem, _url, _reload);
+            new KDirLister::Private::CachedItemsJob(lister, _url, _reload);
 
         } else {
             // dir not in cache or _reload is true
@@ -205,6 +203,8 @@ bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
 
             itemU = new DirItem(_url, resolved);
             itemsInUse.insert(urlStr, itemU);
+            if (lister->d->autoUpdate)
+                itemU->incAutoUpdate();
 
 //        // we have a limit of MAX_JOBS_PER_LISTER concurrently running jobs
 //        if ( lister->d->numJobs() >= MAX_JOBS_PER_LISTER )
@@ -222,12 +222,12 @@ bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
                 if (lister->d->window)
                     job->ui()->setWindow(lister->d->window);
 
-                connect(job, SIGNAL(entries(KIO::Job *, KIO::UDSEntryList)),
-                        this, SLOT(slotEntries(KIO::Job *, KIO::UDSEntryList)));
-                connect(job, SIGNAL(result(KJob *)),
-                        this, SLOT(slotResult(KJob *)));
-                connect(job, SIGNAL(redirection(KIO::Job *,KUrl)),
-                        this, SLOT(slotRedirection(KIO::Job *,KUrl)));
+                connect(job, SIGNAL(entries(KIO::Job*,KIO::UDSEntryList)),
+                        this, SLOT(slotEntries(KIO::Job*,KIO::UDSEntryList)));
+                connect(job, SIGNAL(result(KJob*)),
+                        this, SLOT(slotResult(KJob*)));
+                connect(job, SIGNAL(redirection(KIO::Job*,KUrl)),
+                        this, SLOT(slotRedirection(KIO::Job*,KUrl)));
 
                 emit lister->started(_url);
             }
@@ -256,72 +256,130 @@ bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
 
         // List existing items in a delayed manner, just like things would happen
         // if we were not using the cache.
-        //kDebug() << "Listing" << itemU->lstItems.count() << "cached items soon";
-        new KDirLister::Private::CachedItemsJob(lister, itemU->lstItems, itemU->rootItem, _url, _reload);
+        if (!itemU->lstItems.isEmpty()) {
+            kDebug() << "Listing" << itemU->lstItems.count() << "cached items soon";
+            new KDirLister::Private::CachedItemsJob(lister, _url, _reload);
+        } else {
+            // The other lister hasn't emitted anything yet. Good, we'll just listen to it.
+            // One problem could be if we have _reload=true and the existing job doesn't, though.
+        }
 
 #ifdef DEBUG_CACHE
         printDebug();
 #endif
     }
 
-    // automatic updating of directories
-    if (lister->d->autoUpdate)
-        itemU->incAutoUpdate();
-
     return true;
 }
 
+KDirLister::Private::CachedItemsJob* KDirLister::Private::cachedItemsJobForUrl(const KUrl& url) const
+{
+    Q_FOREACH(CachedItemsJob* job, m_cachedItemsJobs) {
+	if (job->url() == url)
+	    return job;
+    }
+    return 0;
+}
+
+KDirLister::Private::CachedItemsJob::CachedItemsJob(KDirLister* lister, const KUrl& url, bool reload)
+  : KJob(lister),
+    m_lister(lister), m_url(url),
+    m_reload(reload), m_emitCompleted(true)
+{
+    //kDebug() << "Creating CachedItemsJob" << this << "for lister" << lister << url;
+    if (lister->d->cachedItemsJobForUrl(url)) {
+      kWarning(7004) << "Lister" << lister << "has a cached items job already for" << url;
+    }
+    lister->d->m_cachedItemsJobs.append(this);
+    setAutoDelete(true);
+    start();
+}
+
+// Called by start() via QueuedConnection
 void KDirLister::Private::CachedItemsJob::done()
 {
-    //kDebug() << "lister" << m_lister << "says" << m_lister->d->m_cachedItemsJob << "this=" << this;
-    Q_ASSERT(m_lister->d->m_cachedItemsJob == this);
-    kDirListerCache->emitItemsFromCache(m_lister, m_items, m_rootItem, m_url, m_reload, m_emitCompleted);
+    if (!m_lister) // job was already killed, but waiting deletion due to deleteLater
+        return;
+    kDirListerCache->emitItemsFromCache(this, m_lister, m_url, m_reload, m_emitCompleted);
     emitResult();
 }
 
-void KDirListerCache::emitItemsFromCache(KDirLister* lister, const KFileItemList& items, const KFileItem& rootItem, const KUrl& _url, bool _reload, bool _emitCompleted)
+bool KDirLister::Private::CachedItemsJob::doKill()
 {
-    lister->d->m_cachedItemsJob = 0;
+    //kDebug(7004) << this;
+    kDirListerCache->forgetCachedItemsJob(this, m_lister, m_url);
+    if (!property("_kdlc_silent").toBool()) {
+        emit m_lister->canceled(m_url);
+        emit m_lister->canceled();
+    }
+    m_lister = 0;
+    return true;
+}
 
+void KDirListerCache::emitItemsFromCache(KDirLister::Private::CachedItemsJob* cachedItemsJob, KDirLister* lister, const KUrl& _url, bool _reload, bool _emitCompleted)
+{
     const QString urlStr = _url.url();
-    DirItem *itemU = kDirListerCache->itemsInUse.value(urlStr);
-    Q_ASSERT(itemU); // hey we're listing that dir, so this can't be 0, right?
-
     KDirLister::Private* kdl = lister->d;
-
     kdl->complete = false;
 
-    if ( kdl->rootFileItem.isNull() && kdl->url == _url )
-        kdl->rootFileItem = rootItem;
+    DirItem *itemU = kDirListerCache->itemsInUse.value(urlStr);
+    if (!itemU) {
+        kWarning(7004) << "Can't find item for directory" << urlStr << "anymore";
+    } else {
+        const KFileItemList items = itemU->lstItems;
+        const KFileItem rootItem = itemU->rootItem;
+        _reload = _reload || !itemU->complete;
 
-    //kDebug(7004) << "emitting" << items.count() << "for lister" << lister;
-    kdl->addNewItems(_url, items);
-    kdl->emitItems();
+        if (kdl->rootFileItem.isNull() && !rootItem.isNull() && kdl->url == _url) {
+            kdl->rootFileItem = rootItem;
+        }
+        if (!items.isEmpty()) {
+            //kDebug(7004) << "emitting" << items.count() << "for lister" << lister;
+            kdl->addNewItems(_url, items);
+            kdl->emitItems();
+        }
+    }
 
-    KDirListerCacheDirectoryData& dirData = directoryData[urlStr];
-    Q_ASSERT(dirData.listersCurrentlyListing.contains(lister));
+    forgetCachedItemsJob(cachedItemsJob, lister, _url);
 
     // Emit completed, unless we were told not to,
     // or if listDir() was called while another directory listing for this dir was happening,
     // so we "joined" it. We detect that using jobForUrl to ensure it's a real ListJob,
     // not just a lister-specific CachedItemsJob (which wouldn't emit completed for us).
-    if (_emitCompleted && jobForUrl( urlStr ) == 0) {
-
-        Q_ASSERT(!dirData.listersCurrentlyHolding.contains(lister));
-        //kDebug(7004) << "Moving from listing to holding, because emitCompleted is true and no job" << lister << urlStr;
-        dirData.listersCurrentlyHolding.append( lister );
-        dirData.listersCurrentlyListing.removeAll( lister );
+    if (_emitCompleted) {
 
         kdl->complete = true;
         emit lister->completed( _url );
         emit lister->completed();
 
-        if ( _reload || !itemU->complete ) {
+        if ( _reload ) {
             updateDirectory( _url );
         }
     }
 }
 
+void KDirListerCache::forgetCachedItemsJob(KDirLister::Private::CachedItemsJob* cachedItemsJob, KDirLister* lister, const KUrl& _url)
+{
+    // Modifications to data structures only below this point;
+    // so that addNewItems is called with a consistent state
+
+    const QString urlStr = _url.url();
+    lister->d->m_cachedItemsJobs.removeAll(cachedItemsJob);
+
+    KDirListerCacheDirectoryData& dirData = directoryData[urlStr];
+    Q_ASSERT(dirData.listersCurrentlyListing.contains(lister));
+
+    KIO::ListJob *listJob = jobForUrl(urlStr);
+    if (!listJob) {
+        Q_ASSERT(!dirData.listersCurrentlyHolding.contains(lister));
+        //kDebug(7004) << "Moving from listing to holding, because no more job" << lister << urlStr;
+        dirData.listersCurrentlyHolding.append( lister );
+        dirData.listersCurrentlyListing.removeAll( lister );
+    } else {
+        //kDebug(7004) << "Still having a listjob" << listJob << ", so not moving to currently-holding.";
+    }
+}
+
 bool KDirListerCache::validUrl( const KDirLister *lister, const KUrl& url ) const
 {
   if ( !url.isValid() )
@@ -352,49 +410,38 @@ void KDirListerCache::stop( KDirLister *lister, bool silent )
 #ifdef DEBUG_CACHE
     //printDebug();
 #endif
-    //kDebug(7004) << "lister: " << lister;
-    bool stopped = false;
+    //kDebug(7004) << "lister:" << lister << "silent=" << silent;
 
+    const KUrl::List urls = lister->d->lstDirs;
+    Q_FOREACH(const KUrl& url, urls) {
+        stopListingUrl(lister, url, silent);
+    }
+    
+#if 0 // test code
     QHash<QString,KDirListerCacheDirectoryData>::iterator dirit = directoryData.begin();
     const QHash<QString,KDirListerCacheDirectoryData>::iterator dirend = directoryData.end();
     for( ; dirit != dirend ; ++dirit ) {
         KDirListerCacheDirectoryData& dirData = dirit.value();
-        if ( dirData.listersCurrentlyListing.removeAll(lister) ) { // contains + removeAll in one go
-            // lister is listing url
-            const QString url = dirit.key();
-
-            //kDebug(7004) << " found lister" << lister << "in list - for" << url;
-            stopLister(lister, url, dirData, silent);
-            stopped = true;
+        if (dirData.listersCurrentlyListing.contains(lister)) {
+            kDebug(7004) << "ERROR: found lister" << lister << "in list - for" << dirit.key();
+            Q_ASSERT(false);
         }
     }
-
-    if (lister->d->m_cachedItemsJob) {
-        delete lister->d->m_cachedItemsJob;
-        lister->d->m_cachedItemsJob = 0;
-        stopped = true;
-    }
-
-    if ( stopped ) {
-        if (!silent) {
-            emit lister->canceled();
-        }
-        lister->d->complete = true;
-    }
-
-    // this is wrong if there is still an update running!
-    //Q_ASSERT( lister->d->complete );
+#endif
 }
 
-void KDirListerCache::stop(KDirLister *lister, const KUrl& _u, bool silent)
+void KDirListerCache::stopListingUrl(KDirLister *lister, const KUrl& _u, bool silent)
 {
     KUrl url(_u);
     url.adjustPath( KUrl::RemoveTrailingSlash );
     const QString urlStr = url.url();
 
-    if (lister->d->m_cachedItemsJob && lister->d->m_cachedItemsJob->url() == url) {
-        delete lister->d->m_cachedItemsJob;
-        lister->d->m_cachedItemsJob = 0;
+    KDirLister::Private::CachedItemsJob* cachedItemsJob = lister->d->cachedItemsJobForUrl(url);
+    if (cachedItemsJob) {
+        if (silent) {
+            cachedItemsJob->setProperty("_kdlc_silent", true);
+        }
+        cachedItemsJob->kill(); // removes job from list, too
     }
 
     // TODO: consider to stop all the "child jobs" of url as well
@@ -404,34 +451,42 @@ void KDirListerCache::stop(KDirLister *lister, const KUrl& _u, bool silent)
     if (dirit == directoryData.end())
         return;
     KDirListerCacheDirectoryData& dirData = dirit.value();
-    if ( dirData.listersCurrentlyListing.removeAll(lister) ) { // contains + removeAll in one go
-
+    if (dirData.listersCurrentlyListing.contains(lister)) {
         //kDebug(7004) << " found lister" << lister << "in list - for" << urlStr;
-        stopLister(lister, urlStr, dirData, silent);
-
-        if ( lister->d->numJobs() == 0 ) {
-            lister->d->complete = true;
-            // we killed the last job for lister
+        if (dirData.listersCurrentlyListing.count() == 1) {
+            // This was the only dirlister interested in the list job -> kill the job
+            stopListJob(urlStr, silent);
+        } else {
+            // Leave the job running for the other dirlisters, just unsubscribe us.
+            dirData.listersCurrentlyListing.removeAll(lister);
             if (!silent) {
                 emit lister->canceled();
+                emit lister->canceled(url);
             }
-            //kDebug(7004) << "Entry now being listed by" << dirData.listersCurrentlyListing;
         }
     }
 }
 
-// Helper for both stop() methods
-void KDirListerCache::stopLister(KDirLister* lister, const QString& url, KDirListerCacheDirectoryData& dirData, bool silent)
+// Helper for stop() and stopListingUrl()
+void KDirListerCache::stopListJob(const QString& url, bool silent)
 {
-    // Let's just leave the job running.
+    // Old idea: if it's an update job, let's just leave the job running.
     // After all, update jobs do run for "listersCurrentlyHolding",
     // so there's no reason to kill them just because @p lister is now a holder.
 
-    // Move lister to listersCurrentlyHolding
-    dirData.listersCurrentlyHolding.append(lister);
+    // However it could be a long-running non-local job (e.g. filenamesearch), which
+    // the user wants to abort, and which will never be used for updating...
+    // And in any case slotEntries/slotResult is not meant to be called by update jobs.
+    // So, change of plan, let's kill it after all, in a way that triggers slotResult/slotUpdateResult.
 
-    if (!silent)
-        emit lister->canceled(KUrl(url));
+    KIO::ListJob *job = jobForUrl(url);
+    if (job) {
+        //kDebug() << "Killing list job" << job << "for" << url;
+        if (silent) {
+            job->setProperty("_kdlc_silent", true);
+        }
+        job->kill(KJob::EmitResult);
+    }
 }
 
 void KDirListerCache::setAutoUpdate( KDirLister *lister, bool enable )
@@ -461,6 +516,7 @@ void KDirListerCache::forgetDirs( KDirLister *lister )
     const KUrl::List lstDirsCopy = lister->d->lstDirs;
     lister->d->lstDirs.clear();
 
+    //kDebug() << "Iterating over dirs" << lstDirsCopy;
     for ( KUrl::List::const_iterator it = lstDirsCopy.begin();
           it != lstDirsCopy.end(); ++it ) {
         forgetDirs( lister, *it, false );
@@ -502,6 +558,7 @@ void KDirListerCache::forgetDirs( KDirLister *lister, const KUrl& _url, bool not
 
     DirItem *item = itemsInUse.value(urlStr);
     Q_ASSERT(item);
+    bool insertIntoCache = false;
 
     if ( dirData.listersCurrentlyHolding.isEmpty() && dirData.listersCurrentlyListing.isEmpty() ) {
         // item not in use anymore -> move into cache if complete
@@ -528,10 +585,12 @@ void KDirListerCache::forgetDirs( KDirLister *lister, const KUrl& _url, bool not
             emit lister->clear( url );
         }
 
-        if ( item->complete ) {
-            kDebug(7004) << lister << " item moved into cache: " << url;
-            itemsCached.insert( urlStr, item );
-
+        insertIntoCache = item->complete;
+        if (insertIntoCache) {
+            // TODO(afiestas): remove use of KMountPoint+manually_mounted and port to Solid:
+            // 1) find Volume for the local path "item->url.toLocalFile()" (which could be anywhere
+            // under the mount point) -- probably needs a new operator in libsolid query parser
+            // 2) [**] becomes: if (Drive is hotpluggable or Volume is removable) "set to dirty" else "keep watch"
             const KMountPoint::List possibleMountPoints = KMountPoint::possibleMountPoints(KMountPoint::NeedMountOptions);
 
             // Should we forget the dir for good, or keep a watch on it?
@@ -554,7 +613,7 @@ void KDirListerCache::forgetDirs( KDirLister *lister, const KUrl& _url, bool not
                 }
             }
 
-            if ( isManuallyMounted || containsManuallyMounted )
+            if ( isManuallyMounted || containsManuallyMounted ) // [**]
             {
                 kDebug(7004) << "Not adding a watch on " << item->url << " because it " <<
                     ( isManuallyMounted ? "is manually mounted" : "contains a manually mounted subdir" );
@@ -572,6 +631,12 @@ void KDirListerCache::forgetDirs( KDirLister *lister, const KUrl& _url, bool not
 
     if ( item && lister->d->autoUpdate )
         item->decAutoUpdate();
+
+    // Inserting into QCache must be done last, since it might delete the item
+    if (item && insertIntoCache) {
+        kDebug(7004) << lister << "item moved into cache:" << url;
+        itemsCached.insert(urlStr, item);
+    }
 }
 
 void KDirListerCache::updateDirectory( const KUrl& _dir )
@@ -612,11 +677,11 @@ void KDirListerCache::updateDirectory( const KUrl& _dir )
         // Emit any cached items.
         // updateDirectory() is about the diff compared to the cached items...
         Q_FOREACH(KDirLister *kdl, listers) {
-            if (kdl->d->m_cachedItemsJob) {
-                KDirLister::Private::CachedItemsJob* job = kdl->d->m_cachedItemsJob;
-                job->setEmitCompleted(false);
-                job->done(); // sets kdl->d->m_cachedItemsJob to 0
-                delete job;
+	    KDirLister::Private::CachedItemsJob* cachedItemsJob = kdl->d->cachedItemsJobForUrl(_dir);
+            if (cachedItemsJob) {
+                cachedItemsJob->setEmitCompleted(false);
+                cachedItemsJob->done(); // removes from cachedItemsJobs list
+                delete cachedItemsJob;
                 killed = true;
             }
         }
@@ -631,7 +696,7 @@ void KDirListerCache::updateDirectory( const KUrl& _dir )
         kWarning() << "listers for" << _dir << "=" << listers;
         kWarning() << "job=" << job;
         Q_FOREACH(KDirLister *kdl, listers) {
-            kDebug() << "lister" << kdl << "m_cachedItemsJob=" << kdl->d->m_cachedItemsJob;
+            kDebug() << "lister" << kdl << "m_cachedItemsJobs=" << kdl->d->m_cachedItemsJobs;
         }
 #ifndef NDEBUG
         printDebug();
@@ -642,10 +707,10 @@ void KDirListerCache::updateDirectory( const KUrl& _dir )
     job = KIO::listDir( _dir, KIO::HideProgressInfo );
     runningListJobs.insert( job, KIO::UDSEntryList() );
 
-    connect( job, SIGNAL(entries( KIO::Job *, const KIO::UDSEntryList & )),
-             this, SLOT(slotUpdateEntries( KIO::Job *, const KIO::UDSEntryList & )) );
-    connect( job, SIGNAL(result( KJob * )),
-             this, SLOT(slotUpdateResult( KJob * )) );
+    connect( job, SIGNAL(entries(KIO::Job*,KIO::UDSEntryList)),
+             this, SLOT(slotUpdateEntries(KIO::Job*,KIO::UDSEntryList)) );
+    connect( job, SIGNAL(result(KJob*)),
+             this, SLOT(slotUpdateResult(KJob*)) );
 
     kDebug(7004) << "update started in" << _dir;
 
@@ -777,7 +842,7 @@ void KDirListerCache::slotFilesAdded( const QString &dir /*url*/ ) // from KDirN
     KUrl urlDir(dir);
     kDebug(7004) << urlDir; // output urls, not qstrings, since they might contain a password
     if (urlDir.isLocalFile()) {
-        Q_FOREACH(const QString& u, directoriesForCanonicalPath(urlDir.path())) {
+        Q_FOREACH(const QString& u, directoriesForCanonicalPath(urlDir.toLocalFile())) {
             updateDirectory(KUrl(u));
         }
     } else {
@@ -1009,7 +1074,7 @@ void KDirListerCache::slotFileDirty( const QString& path )
     KUrl url(path);
     url.adjustPath(KUrl::RemoveTrailingSlash);
     if (isDir) {
-        Q_FOREACH(const QString& dir, directoriesForCanonicalPath(url.path())) {
+        Q_FOREACH(const QString& dir, directoriesForCanonicalPath(url.toLocalFile())) {
             handleDirDirty(dir);
         }
     } else {
@@ -1112,6 +1177,9 @@ void KDirListerCache::slotEntries( KIO::Job *job, const KIO::UDSEntryList &entri
     KDirListerCacheDirectoryData& dirData = *dit;
     if (dirData.listersCurrentlyListing.isEmpty()) {
         kError(7004) << "Internal error: job is listing" << url << "but directoryData says no listers are currently listing " << urlStr;
+#ifndef NDEBUG
+        printDebug();
+#endif
         Q_ASSERT( !dirData.listersCurrentlyListing.isEmpty() );
         return;
     }
@@ -1168,7 +1236,7 @@ void KDirListerCache::slotEntries( KIO::Job *job, const KIO::UDSEntryList &entri
 void KDirListerCache::slotResult( KJob *j )
 {
 #ifdef DEBUG_CACHE
-  printDebug();
+    //printDebug();
 #endif
 
   Q_ASSERT( j );
@@ -1212,12 +1280,19 @@ void KDirListerCache::slotResult( KJob *j )
     foreach ( KDirLister *kdl, listers )
     {
       kdl->d->jobDone( job );
-      kdl->handleError( job );
-      emit kdl->canceled( jobUrl );
-      if ( kdl->d->numJobs() == 0 )
-      {
+      if (job->error() != KJob::KilledJobError) {
+          kdl->handleError( job );
+      }
+      const bool silent = job->property("_kdlc_silent").toBool();
+      if (!silent) {
+          emit kdl->canceled( jobUrl );
+      }
+
+      if (kdl->d->numJobs() == 0) {
         kdl->d->complete = true;
-        emit kdl->canceled();
+        if (!silent) {
+            emit kdl->canceled();
+        }
       }
     }
   }
@@ -1263,6 +1338,9 @@ void KDirListerCache::slotRedirection( KIO::Job *j, const KUrl& url )
     if ( oldUrl == newUrl ) {
         kDebug(7004) << "New redirection url same as old, giving up.";
         return;
+    } else if (newUrl.isEmpty()) {
+        kDebug(7004) << "New redirection url is empty, giving up.";
+        return;
     }
 
     const QString oldUrlStr = oldUrl.url();
@@ -1412,10 +1490,10 @@ void KDirListerCache::slotRedirection( KIO::Job *j, const KUrl& url )
     // make the job an update job
     job->disconnect( this );
 
-    connect( job, SIGNAL(entries( KIO::Job *, const KIO::UDSEntryList & )),
-             this, SLOT(slotUpdateEntries( KIO::Job *, const KIO::UDSEntryList & )) );
-    connect( job, SIGNAL(result( KJob * )),
-             this, SLOT(slotUpdateResult( KJob * )) );
+    connect( job, SIGNAL(entries(KIO::Job*,KIO::UDSEntryList)),
+             this, SLOT(slotUpdateEntries(KIO::Job*,KIO::UDSEntryList)) );
+    connect( job, SIGNAL(result(KJob*)),
+             this, SLOT(slotUpdateResult(KJob*)) );
 
     // FIXME: autoUpdate-Counts!!
 
@@ -1601,10 +1679,15 @@ void KDirListerCache::slotUpdateResult( KJob * j )
             //don't bother the user
             //kdl->handleError( job );
 
-            emit kdl->canceled( jobUrl );
+            const bool silent = job->property("_kdlc_silent").toBool();
+            if (!silent) {
+                emit kdl->canceled( jobUrl );
+            }
             if ( kdl->d->numJobs() == 0 ) {
                 kdl->d->complete = true;
-                emit kdl->canceled();
+                if (!silent) {
+                    emit kdl->canceled();
+                }
             }
         }
 
@@ -1816,7 +1899,7 @@ void KDirListerCache::deleteDir( const KUrl& dirUrl )
             // we need a copy because stop modifies the list
             QList<KDirLister *> listers = (*dit).listersCurrentlyListing;
             foreach ( KDirLister *kdl, listers )
-                stop( kdl, deletedUrl );
+                stopListingUrl( kdl, deletedUrl );
             // tell listers holding deletedUrl to forget about it
             // this will stop running updates for deletedUrl as well
 
@@ -1906,8 +1989,8 @@ void KDirListerCache::printDebug()
             list += " 0x" + QString::number( (qlonglong)listit, 16 );
         kDebug(7004) << "  " << dit.key() << (*dit).listersCurrentlyListing.count() << "listers:" << list;
         foreach ( KDirLister* listit, (*dit).listersCurrentlyListing ) {
-            if (listit->d->m_cachedItemsJob) {
-                kDebug(7004) << "  Lister" << listit << "has CachedItemsJob" << listit->d->m_cachedItemsJob;
+            if (!listit->d->m_cachedItemsJobs.isEmpty()) {
+                kDebug(7004) << "  Lister" << listit << "has CachedItemsJobs" << listit->d->m_cachedItemsJobs;
             } else if (KIO::ListJob* listJob = jobForUrl(dit.key())) {
                 kDebug(7004) << "  Lister" << listit << "has ListJob" << listJob;
             } else {
@@ -1959,9 +2042,9 @@ KDirLister::KDirLister( QObject* parent )
 
 KDirLister::~KDirLister()
 {
-    //kDebug(7003) << "-KDirLister";
+    //kDebug(7003) << "~KDirLister" << this;
 
-    // Stop all running jobs
+    // Stop all running jobs, remove lister from lists
     if (!kDirListerCache.isDestroyed()) {
         stop();
         kDirListerCache->forgetDirs( this );
@@ -1988,7 +2071,7 @@ void KDirLister::stop()
 
 void KDirLister::stop( const KUrl& _url )
 {
-    kDirListerCache->stop( this, _url );
+    kDirListerCache->stopListingUrl( this, _url );
 }
 
 bool KDirLister::autoUpdate() const
@@ -2074,6 +2157,10 @@ void KDirLister::Private::emitChanges()
     // Mark all items that are currently visible
     Q_FOREACH(const KUrl& dir, lstDirs) {
         KFileItemList* itemList = kDirListerCache->itemsForDir(dir);
+        if (!itemList) {
+            continue;
+        }
+
         KFileItemList::iterator kit = itemList->begin();
         const KFileItemList::iterator kend = itemList->end();
         for (; kit != kend; ++kit) {
@@ -2090,6 +2177,10 @@ void KDirLister::Private::emitChanges()
         KFileItemList deletedItems;
 
         KFileItemList* itemList = kDirListerCache->itemsForDir(dir);
+        if (!itemList) {
+            continue;
+        }
+
         KFileItemList::iterator kit = itemList->begin();
         const KFileItemList::iterator kend = itemList->end();
         for (; kit != kend; ++kit) {
@@ -2173,7 +2264,7 @@ void KDirLister::setMimeFilter( const QStringList& mimeFilter )
         return;
 
     d->prepareForSettingsChange();
-    if (mimeFilter.contains("application/octet-stream")) // all files
+    if (mimeFilter.contains(QLatin1String("application/octet-stream")) || mimeFilter.contains(QLatin1String("all/allfiles"))) // all files
         d->settings.mimeFilter.clear();
     else
         d->settings.mimeFilter = mimeFilter;
@@ -2545,16 +2636,16 @@ void KDirLister::Private::jobStarted( KIO::ListJob *job )
 
 void KDirLister::Private::connectJob( KIO::ListJob *job )
 {
-  m_parent->connect( job, SIGNAL(infoMessage( KJob *, const QString&, const QString& )),
-                     m_parent, SLOT(_k_slotInfoMessage( KJob *, const QString& )) );
-  m_parent->connect( job, SIGNAL(percent( KJob *, unsigned long )),
-                     m_parent, SLOT(_k_slotPercent( KJob *, unsigned long )) );
-  m_parent->connect( job, SIGNAL(totalSize( KJob *, qulonglong )),
-                     m_parent, SLOT(_k_slotTotalSize( KJob *, qulonglong )) );
-  m_parent->connect( job, SIGNAL(processedSize( KJob *, qulonglong )),
-                     m_parent, SLOT(_k_slotProcessedSize( KJob *, qulonglong )) );
-  m_parent->connect( job, SIGNAL(speed( KJob *, unsigned long )),
-                     m_parent, SLOT(_k_slotSpeed( KJob *, unsigned long )) );
+  m_parent->connect( job, SIGNAL(infoMessage(KJob*,QString,QString)),
+                     m_parent, SLOT(_k_slotInfoMessage(KJob*,QString)) );
+  m_parent->connect( job, SIGNAL(percent(KJob*,ulong)),
+                     m_parent, SLOT(_k_slotPercent(KJob*,ulong)) );
+  m_parent->connect( job, SIGNAL(totalSize(KJob*,qulonglong)),
+                     m_parent, SLOT(_k_slotTotalSize(KJob*,qulonglong)) );
+  m_parent->connect( job, SIGNAL(processedSize(KJob*,qulonglong)),
+                     m_parent, SLOT(_k_slotProcessedSize(KJob*,qulonglong)) );
+  m_parent->connect( job, SIGNAL(speed(KJob*,ulong)),
+                     m_parent, SLOT(_k_slotSpeed(KJob*,ulong)) );
 }
 
 void KDirLister::setMainWindow( QWidget *window )
@@ -2644,7 +2735,7 @@ void KDirListerCacheDirectoryData::moveListersWithoutCachedItemsJob(const KUrl&
     QMutableListIterator<KDirLister *> lister_it(listersCurrentlyListing);
     while (lister_it.hasNext()) {
         KDirLister* kdl = lister_it.next();
-        if (!kdl->d->m_cachedItemsJob || kdl->d->m_cachedItemsJob->url() != url) {
+        if (!kdl->d->cachedItemsJobForUrl(url)) {
             // OK, move this lister from "currently listing" to "currently holding".
 
             // Huh? The KDirLister was present twice in listersCurrentlyListing, or was in both lists?
@@ -2654,7 +2745,7 @@ void KDirListerCacheDirectoryData::moveListersWithoutCachedItemsJob(const KUrl&
             }
             lister_it.remove();
         } else {
-            //kDebug(7004) << "Not moving" << kdl << "to listersCurrentlyHolding because it still has job" << kdl->d->m_cachedItemsJob;
+            //kDebug(7004) << "Not moving" << kdl << "to listersCurrentlyHolding because it still has job" << kdl->d->m_cachedItemsJobs;
         }
     }
 }
